#include <main.hpp>
/************************************************
 *              Global Variables                *
 ************************************************/
ESPMegaPRO espmega = ESPMegaPRO();
ESPMegaDisplayOTA internalDisplayOTA = ESPMegaDisplayOTA();
Button rightOnButton(&espmega.inputs, RIGHT_ON_BUTTON_PIN);
Button rightOffButton(&espmega.inputs, RIGHT_OFF_BUTTON_PIN);
Button leftOnButton(&espmega.inputs, LEFT_ON_BUTTON_PIN);
Button leftOffButton(&espmega.inputs, LEFT_OFF_BUTTON_PIN);

/************************************************
 *            End of Global Variables           *
 ************************************************/

void setup()
{
  // Initialize both displayports
  INTERNAL_DISPLAY_UART.begin(INTERNAL_DISPLAY_BAUDRATE);
  ESP_LOGI("CUD IoT OS", "Starting Initialization");
  send_stop_bytes(INTERNAL_DISPLAY_UART);
  INTERNAL_DISPLAY_UART.print("rest");
  send_stop_bytes(INTERNAL_DISPLAY_UART);
  // Wait for the display to reset
  vTaskDelay(1000 / portTICK_PERIOD_MS);
  // Initialize the ESPMegaPRO
  ESP_LOGV("CUD IoT OS", "Initializing ESPMegaPRO");
  espmega.begin();
  // Initialize IoT Modules
  ESP_LOGV("CUD IoT OS", "Initializing IoT Modules");
  espmega.setTimezone("ICT-7");
  espmega.enableIotModule();
  // Setup Ethernet
  ESP_LOGV("CUD IoT OS", "Initializing Ethernet");
  ETH.begin();
  ESP_LOGD("CUD IoT OS", "Binding Ethernet Interface");
  espmega.iot->bindEthernetInterface(&ETH);
  // Connect to Network
  ESP_LOGV("CUD IoT OS", "Connecting to Network");
  espmega.iot->loadNetworkConfig();
  espmega.iot->connectNetwork();
  // Connect to the MQTT Broker
  ESP_LOGV("CUD IoT OS", "Connecting to MQTT Broker");
  espmega.iot->loadMqttConfig();
  vTaskDelay(3000 / portTICK_PERIOD_MS); // Wait for the network to stabilize
  espmega.iot->connectToMqtt();
  // Enable Web Server
  ESP_LOGV("CUD IoT OS", "Enabling Web Server");
  espmega.enableWebServer(80);
  // Initialize Internal Display
  ESP_LOGV("CUD IoT OS", "Initializing Internal Display");
  espmega.enableInternalDisplay(&INTERNAL_DISPLAY_UART);
  // Register all cards with iot
  ESP_LOGV("CUD IoT OS", "Registering Cards with IoT");
  espmega.iot->registerCard(0);
  espmega.iot->registerCard(1);
  // Input callbacks
  // This pre-load the input buffers
  // We need to do this to prevent switches that are left on the "on" position from triggering the callback
  espmega.inputs.registerCallback(handle_input_change);
  espmega.iot->registerMqttCallback(handle_mqtt_message);
  internalDisplayOTA.begin("/intdisp", espmega.display, espmega.webServer);
  button_init();
  ESP_LOGI("CUD IoT OS", "Initialization Complete");
}

void button_init()
{
  // Initialize buttons

  // Right On Button
  rightOnButton.setMinShortPressTime(BUTTON_SHORT_PRESS_TIME_MS);
  rightOnButton.setMinLongPressTime(BUTTON_LONG_PRESS_TIME_MS);
  rightOnButton.setShortPressCallback([]()
                                      { 
                                        // Turn on right light
                                        espmega.outputs.digitalWrite(RIGHT_LIGHT_RELAY_PIN, HIGH);
                                       });
  rightOnButton.setLongPressCallback([]()
                                     { 
                                        // Turn on both lights
                                        espmega.outputs.digitalWrite(RIGHT_LIGHT_RELAY_PIN, HIGH);
                                        espmega.outputs.digitalWrite(LEFT_LIGHT_RELAY_PIN, HIGH);
                                      });
  rightOnButton.begin();

  // Right Off Button
  rightOffButton.setMinShortPressTime(BUTTON_SHORT_PRESS_TIME_MS);
  rightOffButton.setMinLongPressTime(BUTTON_LONG_PRESS_TIME_MS);
  rightOffButton.setShortPressCallback([]()
                                       { 
                                        // Turn off right light
                                        espmega.outputs.digitalWrite(RIGHT_LIGHT_RELAY_PIN, LOW);
                                        });
  rightOffButton.setLongPressCallback([]()
                                      { 
                                        // Turn off both lights
                                        espmega.outputs.digitalWrite(RIGHT_LIGHT_RELAY_PIN, LOW);
                                        espmega.outputs.digitalWrite(LEFT_LIGHT_RELAY_PIN, LOW);
                                       });
  rightOffButton.begin();

  // Left On Button
  leftOnButton.setMinShortPressTime(BUTTON_SHORT_PRESS_TIME_MS);
  leftOnButton.setMinLongPressTime(BUTTON_LONG_PRESS_TIME_MS);
  leftOnButton.setShortPressCallback([]()
                                     { 
                                        // Turn on left light
                                        espmega.outputs.digitalWrite(LEFT_LIGHT_RELAY_PIN, HIGH);
                                      });
  leftOnButton.setLongPressCallback([]()
                                    { 
                                      // Turn on both lights
                                      espmega.outputs.digitalWrite(RIGHT_LIGHT_RELAY_PIN, HIGH);
                                      espmega.outputs.digitalWrite(LEFT_LIGHT_RELAY_PIN, HIGH);
                                     });
  leftOnButton.begin();

  // Left Off Button
  leftOffButton.setMinShortPressTime(BUTTON_SHORT_PRESS_TIME_MS);
  leftOffButton.setMinLongPressTime(BUTTON_LONG_PRESS_TIME_MS);
  leftOffButton.setShortPressCallback([]()
                                      { 
                                        // Turn off left light
                                        espmega.outputs.digitalWrite(LEFT_LIGHT_RELAY_PIN, LOW);
                                       });
  leftOffButton.setLongPressCallback([]()
                                     { 
                                      // Turn off both lights
                                      espmega.outputs.digitalWrite(RIGHT_LIGHT_RELAY_PIN, LOW);
                                      espmega.outputs.digitalWrite(LEFT_LIGHT_RELAY_PIN, LOW);
                                      });
  leftOffButton.begin();
}

void loop()
{
  espmega.loop();
}

void send_stop_bytes(HardwareSerial &uart)
{
  uart.write(0xFF);
  uart.write(0xFF);
  uart.write(0xFF);
}

void handle_input_change(uint8_t pin, bool state)
{
  // Do nothing
}

void handle_mqtt_message(char *topic, char *payload)
{
  // Do nothing
}